from selenium.webdriver.common.action_chains import ActionChains  # Type text without specific Element


def sendkeys(driver, keys):  # send keys without specific Element
    actions = ActionChains(driver)
    actions.send_keys(str(keys))
    actions.perform()



import time  # for time.sleep()
import traceback  # print exception
import urllib  # for url parsing
import warnings
from typing import Dict  # define types in functions

from selenium import webdriver
from selenium_profiles.scripts import profiles
from selenium_profiles.utils.colab_utils import is_colab
from selenium_profiles.scripts.cdp_tools import cdp_tools
from selenium_profiles.scripts.driver_utils import sendkeys
from selenium_profiles.scripts import undetected
from selenium_profiles.utils.utils import sel_profiles_path, find_chrome_executable  # read txt files


# noinspection PyPep8Naming,GrazieInspection
class driver(object):
    def __init__(self):
        # initial attributes
        # noinspection SpellCheckingInspection
        self.returnnavigator = None
        self.profile = None
        self.driver = None
        self.cdp_tools = None
        self.options = None

        self.profiles = profiles

    def start(self, profile: Dict[str, dict or list], uc_driver: bool = False):
        self.profile = profile

        if self.profile["plugins"]["selenium-wire"]:
            warnings.warn("Selenium-wire not supported yet, ignoring")

        if is_colab():  # google-colab doesn't support sandbox!
            if self.profile["browser"]["sandbox"] is True:
                warnings.warn('Google-colab doesn\'t work with sandbox enabled yet, disabling..')
            self.profile["browser"]["sandbox"] = False

        if uc_driver:
            try:
                # noinspection PyUnresolvedReferences
                import undetected_chromedriver as uc  # undetected chromedriver
            except ImportError:
                warnings.warn('undetected-chromedriver not installed. Installing..')
                import os
                from selenium_profiles.utils.colab_utils import patch_uc
                os.system('pip install undetected-chromedriver')
                patch_uc()
            # noinspection PyUnboundLocalVariable
            self.options = uc.ChromeOptions()  # selenium.webdriver options, https://peter.sh/experiments/chromium-command-line-switches/
        else:
            self.options = webdriver.ChromeOptions()  # selenium.webdriver options, https://peter.sh/experiments/chromium-command-line-switches/

        # options-manager
        self.options = self.profiles.to_options(self.profile, self.options)

        # EXTENSIONS

        # ModHeader
        if self.profile["plugins"]["modheader"]:
            import os
            warnings.warn('Only use modheader when additional Headers needed!')
            if not os.path.isdir(sel_profiles_path() + "files/modheader"):
                warnings.warn('Modheader not installed & extracted in /modheader yet!')
                from selenium_profiles.utils.installer import install_modheader
                install_modheader()
            self.options.add_argument('--load-extension=' + sel_profiles_path() + "files/modheader")

        # Buster
        if self.profile["plugins"]["buster"]:
            import os
            warnings.warn("Buster is deprecated and automating isn't supported!")
            warnings.warn('Only use Buster when Captcha solver needed!')
            if not os.path.isdir(sel_profiles_path() + "files/buster"):
                warnings.warn('Buster not installed & extracted in /buster yet!')
                from selenium_profiles.utils.installer import install_buster
                install_buster()
            self.options.add_argument('--load-extension=' + sel_profiles_path() + "files/buster")

        # ACTUAL START

        if uc_driver:
            # noinspection PyUnboundLocalVariable
            self.driver = uc.Chrome(use_subprocess=True, options=self.options, keep_alive=True,
                                    browser_executable_path=find_chrome_executable())  # start undetected_chromedriver
        else:
            self.options = undetected.config_options(self.options)

            # Actual start of chrome
            self.driver = webdriver.Chrome(options=self.options)  # start selenium webdriver

        self.driver.get('http://motherfuckingwebsite.com/')  # wait browser to start

        self.cdp_tools = cdp_tools(self.driver)

        # execute cdp based on profile
        self.profiles.exec_cdp(self.profile, self.driver)

        self.profiles.exec_js_evaluators(self.profile, self.driver, self.cdp_tools)

        if not uc_driver:
            undetected.exec_cdp(self.driver, self.cdp_tools)

        if profile["plugins"]["modheader"]:
            self.load_header_profiles(profile["plugins"]["modheader"])

        self.driver.profile = profile
        self.add_funcs_to_driver()

        # Return actual driver
        return self.driver

    def add_funcs_to_driver(self):

        self.driver.cdp_tools = self.cdp_tools

        # add my functions to driver
        self.driver.send_keys = sendkeys

        self.driver.get_profile = self.profiles.get_profile
        self.driver.get_navigator = self.profiles.get_navigator
        self.driver.navigator2profile = profiles.navigator2profile

        # ModHeader
        self.driver.load_header_profiles = self.load_header_profiles
        self.driver.clear_headers = self.clear_headers

        # Captcha
        self.driver.solve_captcha = self.solve_captcha

        # patch cookie functions
        self.driver.get_cookies = self.cdp_tools.get_cookies
        self.driver.add_cookie = self.cdp_tools.add_cookie
        self.driver.get_cookie = self.cdp_tools.get_cookie
        self.driver.delete_cookie = self.cdp_tools.delete_cookie
        self.driver.delete_all_cookies = self.cdp_tools.delete_all_cookies

    def export_profile(self, to_path=sel_profiles_path() + "files/user_dir"):
        import shutil
        shutil.copytree(self.driver.user_data_dir, to_path)

    def start_no_profile(self, modheader: bool = False, buster: bool = False, user_dir: str = None,
                         arguments: list = None):  # start minimal driver without profile
        import undetected_chromedriver as uc  # undetected chromedriver
        options = uc.ChromeOptions()
        # additional options
        if arguments:
            for arg in arguments:
                options.add_argument(arg)

        options.arguments.extend(["--no-sandbox", "--test-type"])

        if user_dir:
            options.add_argument(r"--user-data-dir=" + user_dir)

        # ModHeader extension options
        if modheader:
            import os
            warnings.warn('Only use modheader when additional Headers needed!')
            if not os.path.isdir(os.getcwd() + "/modheader"):
                warnings.warn('Modheader not installed & extracted in /modheader yet!')
                from selenium_profiles.utils.installer import install_modheader
                install_modheader()
            options.add_argument('--load-extension=' + os.getcwd() + "/modheader")

        # Buster extension options
        if buster:
            import os
            warnings.warn("Buster is deprecated and automating isn't supported!")
            warnings.warn('Only use Buster when Captcha solver needed!')
            if not os.path.isdir(os.getcwd() + "/buster"):
                warnings.warn('Buster not installed & extracted in /buster yet!')
                from selenium_profiles.utils.installer import install_buster
                install_buster()
            options.add_argument('--load-extension=' + os.getcwd() + "/buster")

        self.driver = uc.Chrome(use_subprocess=True, options=options, keep_alive=True,
                                browser_executable_path=find_chrome_executable())  # start undetected_chromedriver

        self.add_funcs_to_driver()

        return self.driver

    # noinspection PyUnresolvedReferences
    def load_header_profiles(self, profile: str):
        if self.profile["plugins"]["modheader"]:
            self.driver.modheader_url = 'https://webdriver.modheader.com/load?profile=' + urllib.parse.quote(
                profile[1:-1], safe='')
            self.driver.get(
                'https://webdriver.modheader.com/load?profile=' + urllib.parse.quote(profile[1:-1], safe=''))
        else:
            warnings.warn('ModHeader needs to be enabled for custom headers!')

    def clear_headers(self):
        if self.profile["plugins"]["modheader"]:
            self.driver.get('https://webdriver.modheader.com/clear')
        else:
            warnings.warn('ModHeader needs to be enabled for custom headers!')

    def solve_captcha(self):
        from selenium.webdriver.common.by import By  # locate elements
        from selenium.webdriver.support import expected_conditions as EC
        from selenium.webdriver.support.ui import WebDriverWait

        # noinspection PyBroadException
        try:
            wait = WebDriverWait(self.driver, 5)  # driver 5s timeout
            #  click on captcha
            wait.until(EC.frame_to_be_available_and_switch_to_it(
                (By.CSS_SELECTOR, "iframe[name^='a-'][src^='https://www.google.com/recaptcha/api2/anchor?']")))
            wait.until(EC.element_to_be_clickable((By.XPATH, "//span[@id='recaptcha-anchor']"))).click()
            self.driver.switch_to.default_content()
            # click on audio
            WebDriverWait(self.driver, 4).until(EC.frame_to_be_available_and_switch_to_it(
                (By.CSS_SELECTOR, "iframe[name^='c-'][src^='https://www.google.com/recaptcha/api2/bframe?']")))
            time.sleep(2)
            self.driver.find_element(By.XPATH, '//*[@id="recaptcha-audio-button"]').click()
            self.driver.switch_to.default_content()
        except:
            # check "try again later"
            WebDriverWait(self.driver, 4).until(EC.frame_to_be_available_and_switch_to_it(
                (By.CSS_SELECTOR, "iframe[name^='c-'][src^='https://www.google.com/recaptcha/api2/bframe?']")))
            self.driver.find_element(By.CSS_SELECTOR,
                                     "a[href='https://developers.google.com/recaptcha/docs/faq#my-computer-or-network-may-be-sending-automated-queries']")
            traceback.print_exc()
            warnings.warn("Captcha could not be solved!")
            self.driver.switch_to.default_content()



def return_profile(platform: str):
    from selenium_profiles.utils.utils import read_json
    profile = read_json(filename="profiles/default.json")
    return profile[platform]  # yet supported: "Android", "Windows"


# noinspection PyPep8Naming
def Windows():
    return return_profile("Windows")


# noinspection PyPep8Naming
def Android():
    return return_profile("Android")



def to_options(profile: dict, options):
    import warnings

    mobile = profile['device']['mobile']

    size = profile["browser"]["window_size"]
    options.add_argument("--window-size=" + str(size["x"]) + "," + str(size["y"]))
    options.add_argument('--user-agent=' + profile["device"]["agent_override"]["userAgent"])

    if not profile["browser"]["sandbox"]:
        options.arguments.extend(["--no-sandbox", "--test-type"])
        options.add_argument('--enable-privacy-sandbox-ads-apis')
        warnings.warn('More likely to get detected with sandbox set to False!')
    if profile["browser"]["headless"]:
        options.add_argument('--headless=chrome')
        options.add_argument("--blink-settings=imagesEnabled=false")  # disable loading Images
    if profile["browser"]["touch_events"]:
        options.add_argument("--touch-events=enabled")
    if profile["browser"]["app"]:
        options.add_argument('--app')
        options.add_argument('--force-app-mode')
    if profile["device"]["touch_device"]:
        options.add_argument("--touch-events=enabled")  # enable touch events
    if profile["browser"]["inkognito"]:
        if (profile["plugins"]["modheader"] is False) and (profile["plugins"]["buster"] is False):
            options.add_argument("--incognito")
        else:
            warnings.warn('Incognito not working with Extensions!, disabling Incognito')
    if not profile["device"]['hardware_accel']:
        options.add_argument('--disable-gpu')
        options.add_argument('--override-use-software-gl-for-tests')
    if mobile:
        options.add_argument('--arc-availability=officially-supported')
        options.add_argument('--force-device-ownership')
        options.add_argument('--enable-features=enable-nacl')
        options.add_argument('--use-mobile-user-agent')
        options.add_argument('--enable-touchview')
    if profile["browser"]["proxy"]:
        options.add_argument('--proxy-server=socks5://' + profile["browser"]["proxy"])
        print('proxy= "' + profile["browser"]["proxy"] + '"')

    # additional options and capabilities from profile
    if len(profile["chromeoptions"]["arguments"]) > 0:
        for arg in profile["chromeoptions"]["arguments"]:
            options.add_argument(arg)
    if len(profile["chromeoptions"]["capabilities"][0]) > 0:
        for cap in profile["chromeoptions"]["capabilities"]:
            options.set_capability(cap[0], cap[1])

    return options


def exec_cdp(profile: dict, driver):
    mobile = profile['device']['mobile']

    from selenium_profiles.scripts.cdp_tools import cdp_tools
    cdp_tools = cdp_tools(driver)

    # noinspection PyUnusedLocal
    x = driver.execute_cdp_cmd('Emulation.setIdleOverride', {'isUserActive': True, 'isScreenUnlocked': True})

    # EMULATION
    cdp_tools.set_emulation(profile["device"]["emulation"], enabled=mobile)
    cdp_tools.set_useragent(profile["device"]["agent_override"])

    # TOUCH
    cdp_tools.pointer_as_touch(mobile, profile["browser"]["pointer_as_touch"])
    cdp_tools.set_touchpoints(profile["device"]["touch_device"], maxpoints=profile["device"]["maxtouchpoints"])

    # BROWSER
    cdp_tools.set_darkmode(enabled=profile["browser"]["darkmode"], mobile=mobile)

    # additional cdp_cmd commands from profile
    if len(profile["cdp_cmd"][0]) > 0:
        for args in profile["cdp_cmd"]:
            driver.execute_cdp_cmd(args[0], args[1])


def exec_js_evaluators(profile: dict, driver, cdp_tools=None):
    if cdp_tools:
        do_return = False
    else:
        from selenium_profiles.scripts.cdp_tools import cdp_tools
        cdp_tools = cdp_tools(driver)
        do_return = True

    mobile = profile['device']['mobile']

    # remove plugins when mobile
    # if mobile:
    #    cdp_tools.define_prop_on_new_document("navigator", "plugins", []) # works, but functions missing..

    cdp_tools.define_prop_on_new_document("navigator", "platform", profile["device"]["agent_override"]["platform"])

    # additional evaluate on new document
    if profile["evaluate_on_new_document"]:
        print('identifier for ""evaluate_on_new_document" in profile is :' + str(
            cdp_tools.evaluate_on_new_document(profile["evaluate_on_new_document"])))

    if do_return:
        return cdp_tools.evaluate_on_document_identifiers


# convert exported "navigator" to Profile
def navigator2profile(navigator, filename=None) -> (dict or str, str):
    import json
    from selenium_profiles.utils.utils import write_json
    import traceback

    def replace_none(var, replace_with):
        if var is not None:
            return var
        else:
            return replace_with

    if navigator:
        if type(navigator) is str:
            navigator = json.loads(navigator)
        try:
            navigator["metrics"]["orientation"] = replace_none(navigator["metrics"]["orientation"], "portrait-primary")
            navigator["metrics"]["angle"] = replace_none(navigator["metrics"]["angle"], 0)

            empty_profile = {}
            # Map & convert Values to Profile
            empty_profile.update({'device': {'mobile': navigator["useragentdata"]["mobile"],
                                             'model': '',
                                             'language': navigator["language"],
                                             'touch_device': navigator["metrics"]["maxtouchpoints"] > 0,
                                             'maxtouchpoints': navigator["metrics"]["maxtouchpoints"],
                                             'hardware_accel': False,
                                             'emulation': {'width': navigator["metrics"]["width"],
                                                           'height': navigator["metrics"]["height"],
                                                           'deviceScaleFactor': navigator["metrics"]["scale_factor"],
                                                           'screenOrientation': {
                                                               'type': navigator["metrics"]["orientation"],
                                                               'angle': navigator["metrics"]["angle"]}},
                                             'agent_override': {'platform': navigator["platform"],
                                                                'userAgent': navigator["useragent"],
                                                                'userAgentMetadata': {
                                                                    'brands': navigator["useragentdata"]["brands"],
                                                                    'fullVersionList': navigator["useragentdata"][
                                                                        "brands"],
                                                                    'fullVersion':
                                                                        navigator["useragentdata"]["brands"][0][
                                                                            "version"] + '.0.0000.00',
                                                                    'platform': navigator["useragentdata"]["platform"],
                                                                    'platformVersion': '10.0.0', 'architecture': 'x86',
                                                                    'model': '',
                                                                    'mobile': navigator["useragentdata"]["mobile"],
                                                                    'bitness': '',
                                                                    'wow64': navigator["wow64"]}}},
                                  'browser': {'sandbox': True,
                                              'inkognito': True,
                                              'proxy': None,
                                              'darkmode': navigator["useragentdata"]["mobile"],
                                              'pointer_as_touch': False,
                                              'mobile_view': navigator["useragentdata"]["mobile"],
                                              'app': False,
                                              'touch_events': navigator["metrics"]["maxtouchpoints"] > 0,
                                              'window_size': {'x': navigator["metrics"]["width"],
                                                              'y': navigator["metrics"]["height"]}},
                                  'chromeoptions': {'arguments': [], 'capabilities': []},
                                  'cdp_cmd': [],
                                  'plugins': {'selenium-wire': False, 'modheader': False, 'stealth': False,
                                              "buster": False}})
        except:
            traceback.print_exc()
            raise ValueError("Navigator not converted correctly!")
        if filename:
            write_json(empty_profile, filename=filename)
        return empty_profile
    else:
        raise ValueError("navigator was None or False!")


# get profile from current driver
def get_profile(driver, filename: str = None):
    navigator = get_navigator(driver)
    return navigator2profile(navigator, filename=filename)


# get "navigator", object convertible to profile with "navigator2profile"
def get_navigator(driver):
    import traceback
    import warnings
    from selenium_profiles.utils.utils import read
    js_return_navigator = read('js/get_navigator.js')  # for exporting javascript Variables
    # noinspection PyBroadException
    try:
        returnnavigator = driver.execute_script(js_return_navigator)
        return returnnavigator
    except:
        traceback.print_exc()
        warnings.warn('get_navigator.js executed incorrectly!')



from selenium_profiles.utils.utils import read


def exec_cdp(driver, cdp_tools=None):
    if cdp_tools:
        do_return = False
    else:
        from selenium_profiles.scripts.cdp_tools import cdp_tools
        cdp_tools = cdp_tools(driver)
        do_return = True

    # set webdriver js var to false
    cdp_tools.evaluate_on_new_document(read('js/undetected/navigator_webdriver.js'))

    def get_cdc_props():  # if true ==> detectable
        return driver.execute_script(read('js/undetected/get_cdc_props.js'))

    if get_cdc_props():
        cdp_tools.evaluate_on_new_document(read('js/undetected/remove_cdc_props.js'))

        if do_return:
            return cdp_tools.evaluate_on_document_identifiers


def config_options(options):
    options.add_experimental_option("excludeSwitches", ["enable-automation"])
    options.add_experimental_option('useAutomationExtension', False)
    options.arguments.extend(["--disable-blink-features=AutomationControlled", "--disable-blink-features"])

    # suppress welcome
    options.arguments.extend(["--no-default-browser-check", "--no-first-run"])

    return options



import json
import warnings
import os
import selenium_profiles


def sel_profiles_path():
    return os.path.dirname(selenium_profiles.__file__) + "/"


def write(lst: any, filename: str = 'out', encoding: str = "utf-8"):  # write list to textfile line by line
    if type(lst) is dict:
        filename = filename + '.json'
    else:
        filename = filename + '.txt'
    with open(sel_profiles_path() + filename, 'w', encoding=encoding) as fp:
        if type(lst) is list:
            fp.write(str('\n'.join(map(str, lst))))
        elif type(lst) is dict:
            fp.write(json.dumps(lst))
        elif type(lst) is str:
            fp.write(lst)
        else:
            warnings.warn("object not recognised!")
            fp.write(str(lst))
        # fp.write(str(words))
    print('Wrote to "' + filename + '"')


def read(filename: str, encoding: str = "utf-8"):
    with open(sel_profiles_path() + filename, encoding=encoding) as f:
        return f.read()


def read_json(filename: str = 'example.json', encoding: str = "utf-8"):
    with open(sel_profiles_path() + filename, 'r', encoding=encoding) as f:
        return json.load(f)


def write_json(obj: dict or list, filename: str = "out.json", encoding: str = "utf-8"):
    with open(sel_profiles_path() + filename, "w", encoding=encoding) as outfile:
        outfile.write(json.dumps(obj))


def find_chrome_executable():  # needed until https://github.com/ultrafunkamsterdam/undetected-chromedriver/pull/920 gets patched
    import sys
    """
    Finds the chrome, chrome beta, chrome canary, chromium executable
    Returns
    -------
    executable_path :  str
        the full file path to found executable
    """
    candidates = set()
    IS_POSIX = sys.platform.startswith(
        ("darwin", "cygwin", "linux"))  # imported from .patcher in undetected_chromedriver
    if IS_POSIX:
        for item in os.environ.get("PATH").split(os.pathsep):
            for subitem in (
                    "google-chrome",
                    "chromium",
                    "chromium-browser",
                    "chrome",
                    "google-chrome-stable",
            ):
                candidates.add(os.sep.join((item, subitem)))
        if "darwin" in sys.platform:
            candidates.update(
                [
                    "/Applications/Google Chrome.app/Contents/MacOS/Google Chrome",
                    "/Applications/Chromium.app/Contents/MacOS/Chromium",
                ]
            )
    else:
        for item in map(
                os.environ.get, ("PROGRAMFILES", "PROGRAMFILES(X86)", "LOCALAPPDATA", "ProgramW6432")
        ):
            if item:
                for subitem in (
                        "Google/Chrome/Application",
                        "Google/Chrome Beta/Application",
                        "Google/Chrome Canary/Application",
                ):
                    candidates.add(os.sep.join((item, subitem, "chrome.exe")))
    for candidate in candidates:
        if os.path.exists(candidate) and os.access(candidate, os.X_OK):
            return os.path.normpath(candidate)



import unittest

from selenium_profiles.utils.utils import read_json
from selenium_profiles.driver import driver as mydriver
from selenium.webdriver.common.by import By  # locate elements

mydriver = mydriver()


def test_driver(choose: str, headless: bool = True, modheader: str = False, uc_driver=False):
    profile = read_json(filename='profiles\\default.json')
    # noinspection PyGlobalUndefined
    global mydriver
    testprofile = profile[choose]
    testprofile["plugins"]["modheader"] = modheader
    testprofile["browser"]["headless"] = headless
    driver = mydriver.start(testprofile, uc_driver=uc_driver)
    driver.get('https://browserleaks.com/client-hints')
    useragent = driver.find_element(By.XPATH, '//*[@id="content"]/table[1]/tbody/tr/td[2]').accessible_name
    exported_profile = driver.get_profile(driver)
    driver.quit()
    return {"useragent": useragent, "exported_profile": exported_profile}


class Driver(unittest.TestCase):
    # noinspection PyGlobalUndefined
    # Hello World!
    def test_windows(self):
        global mydriver
        output = test_driver('Windows', headless=True)
        self.assertEqual(mydriver.profile['device']['agent_override']['userAgent'],
                         output["useragent"])  # add assertion here

    # noinspection PyGlobalUndefined
    def test_android(self):
        global mydriver
        output = test_driver('Android', headless=False, modheader='[{"headers":[{"enabled":true,"name":"google","value":"\\"x\\""}],"shortTitle":"1","title":"Profile 1","version":2}]')
        self.assertEqual(mydriver.profile['device']['agent_override']['userAgent'],
                         output["useragent"])  # add assertion here


if __name__ == '__main__':
    unittest.main()



import os
import warnings
import requests
import zipfile
import shutil


from selenium_profiles.utils.utils import sel_profiles_path
from selenium_profiles.utils.colab_utils import is_colab


def my_platform():
    import platform
    if is_colab():
        return "Google-Colab"
    else:
        return platform.system()


def installer(dirname, url) -> (str, str):
    if os.path.isdir(sel_profiles_path() + dirname):
        print('Updating ' + dirname + ' ...')
        shutil.rmtree(sel_profiles_path() + dirname)  # remove directory

    r = requests.get(url, allow_redirects=True)
    open(sel_profiles_path() + dirname + '.crx', 'wb').write(r.content)
    extract_del(dirname + '.crx', dirname)


def install_chromedriver(platform: str = my_platform(), chromeversion: int = 108):
    if os.path.isfile(sel_profiles_path() + "files/chromedriver.exe"):
        print(r'Updating "files/chromedriver.exe" ..')

    def write():
        # write .zip
        open(sel_profiles_path() + r'files/chromedriver.zip', 'wb').write(r.content)
        extract_del(r'files/chromedriver.zip', dirname="files/")

    # Versions
    if chromeversion == 108:
        url = 'https://chromedriver.storage.googleapis.com/108.0.5359.22/'
    elif chromeversion == 107:
        url = 'https://chromedriver.storage.googleapis.com/107.0.5304.62/'
    elif chromeversion == 106:
        url = 'https://chromedriver.storage.googleapis.com/106.0.5249.61/'
    else:
        url = None
        warnings.warn("Chromedriver Version " + str(chromeversion) + " not added to installer yet!")

    # Platforms
    if platform == "Windows":
        r = requests.get(url + 'chromedriver_win32.zip')
        write()
    elif platform == "Linux":
        warnings.warn("Linux not tested yet!")
        os.system('apt install chromium-chromedriver;apt install -y xvfb;zip -j /content/chromedriver_linux64.zip /usr/bin/chromedriver')
    elif platform == "Google-Colab":
        from selenium_profiles.utils.colab_utils import patch_uc, update_apts
        update_apts()
        os.system('apt install chromium-chromedriver;apt install -y xvfb;zip -j /content/chromedriver_linux64.zip /usr/bin/chromedriver')
        patch_uc()
    else:
        warnings.warn('Chromedriver installation for "' + platform + '" not supported yet!')


def extract_del(filename, dirname) -> (str, str):
    with zipfile.ZipFile(sel_profiles_path() + filename, 'r') as zObject:
        zObject.extractall(path=sel_profiles_path() + dirname)

    if os.path.isdir(sel_profiles_path() + dirname):
        print(filename + ' extracted to "' + dirname + '" successfully!')
    else:
        raise ValueError(
            filename + ' could not be installed correctly!, "' + dirname + '" doesn\'t exist or isn\'t a directory')
    os.remove(sel_profiles_path() + filename)


def install_modheader(dirname: str = r"files/modheader"):
    installer(dirname,
              'https://github.com/modheader/modheader_selenium/blob/main/chrome-modheader/modheader.crx?raw=true')


def install_buster(dirname: str = r"files/buster", patch_files: bool = False):
    installer(dirname,
              'https://github.com/dessant/buster/releases/download/v1.3.2/buster_captcha_solver_for_humans-1.3.2-chrome.zip')
    if patch_files:
        print('Patching "Buster" extension..')
        warnings.warn("Patch gets detected by Buster!, Not working yet.")
        patch("/files/buster/src/solve/script.js", 'mode:"closed"', 'mode:"open"')
        patch("/files/buster/src/solve/script.js.map", "mode: 'closed'", "mode: 'open'")
        patch("/files/buster/manifest.json", '"notifications",', "")
        print("\"Buster\" extension patched successfully! ")


def patch(filename: str, replace: str, replacewith: str):
    # replace string inside file
    file = open(sel_profiles_path() + filename, "rt")
    patched = file.read().replace(replace, replacewith)
    file.close()
    file = open(sel_profiles_path() + filename, "wt")
    file.write(patched)
    file.close()
    print('Patched "' + filename + '" successfully!')


if __name__ == '__main__':
    install_modheader()
    install_buster(patch_files=False)
    install_chromedriver()



import os
import pathlib
import re
import subprocess
import typing
import warnings


def restart_runtime():
    if is_colab():
        import os
        os.kill(os.getpid(), 9)
    else:
        raise EnvironmentError('Restarting environment necessary..!')


def is_colab():
    import sys
    # noinspection PyBroadException
    try:
        # noinspection PyPackageRequirements
        import google.colab
        cimport = True
    except:
        cimport = False
    if ('google.colab' in sys.modules) and cimport:
        return True
    else:
        return False


def is_jupyter() -> bool:
    """It checks whether a module is run from a Jupyter notebook"""
    try:
        # noinspection PyUnresolvedReferences,PyStatementEffect
        get_ipython
        return True
    except NameError:
        # noinspection PyTypeChecker
        return


def get_module_path(module: str) -> typing.Optional[pathlib.Path]:
    """It gets the absolute path of a Python module"""
    r = subprocess.run(
        ["pip", "show", module],
        capture_output=True
    )

    try:
        r.check_returncode()
    except subprocess.CalledProcessError:
        return None

    stdout = r.stdout.decode()

    try:
        RE_abspath = '\nLocation: (?P<abspath>.*)\n'
        abspath = re.search(RE_abspath, stdout).group('abspath')
    except AttributeError:
        return None

    return pathlib.Path(abspath).absolute()


def patch_uc(src: str = '/content/chromedriver_linux64.zip',
             dst: str = '/tmp/chromedriver_linux64.zip') -> None:
    from selenium_profiles.utils.colab_utils import restart_runtime
    PY_uc = 'undetected_chromedriver'
    """It forces undetected_chromedriver to run the webdriver on Chromium

    It patches 'undetected_chromedriver/patcher.py' to make it work on Google Colab
    by forcing the webdriver to run on Chromium, which has less stringent constraints.

    Parameters
    ----------
    src
        The src Chromium webdriver abspath

    dst
        The dst Chromium webdriver abspath
    """
    if not is_colab():
        warnings.warn('Patching undetected-chromedriver in non-google-colab')

    dirpath = get_module_path(PY_uc)

    if dirpath is None:
        warnings.warn(
            f"Cannot find the {PY_uc} module. "
            f"If you want to use undetected-chromedriver"
            f"you can install it with 'pip install {PY_uc}'\n"
            f"and then run in Python:\n"
            f'from selenium_profiles.utils.colab_utils import patch_uc'
            f'patch_uc()'
        )
        return

    N_patcher_src = dirpath / PY_uc / 'patcher.py'
    O_patcher_src = dirpath / PY_uc / 'patcher_O.py'

    if O_patcher_src.exists():
        return

    with N_patcher_src.open('rt') as f:
        contents = f.read()

        with O_patcher_src.open('wt') as g:
            g.write(contents)

        contents = contents.replace(
            "return urlretrieve(u)[0]",
            "return urlretrieve("
            f"'file://{src}', "
            f"'{dst}'"
            ")[0]"
        )

    with N_patcher_src.open('wt') as f:
        f.write(contents)

    if is_colab():
        restart_runtime()


def update_apts():
    success = os.system("apt-get update >> mytmp")
    with open('mytmp', 'r') as f:
        out = f.read()
    os.remove('mytmp')
    if success == 0:
        return out
    else:
        print(out)
        raise ValueError("Updates not successful!!")


# noinspection PyPep8Naming
class display:
    def __init__(self):
        self.display = None

    def start_display(self, x=1200, y=2200):
        # noinspection PyUnresolvedReferences,PyPackageRequirements
        from pyvirtualdisplay import Display
        self.display = Display(visible=0, size=(x, y))
        self.display.start()

    def stop_display(self):
        self.display.stop()


# noinspection PyUnresolvedReferences,PyPackageRequirements
def showscreen(driver):
    from IPython.display import display as imagerender
    from IPython.display import Image
    driver.save_screenshot('screen.png')
    imagerender(Image('screen.png'))



from typing import Dict, List  # define types in functions
import warnings
import json


# noinspection PyPep8Naming
class cdp_tools(object):
    def __init__(self, driver):
        self.driver = driver

        # initial property
        self.evaluate_on_document_identifiers = {}

    # EMULATION

    def set_emulation(self, emulation, enabled=True) -> (Dict[str, int or Dict[str, str or int or float]], bool):
        emulation.update({"mobile": enabled})
        if enabled:
            warnings.warn('disabling emulation not supported')
            x = self.driver.execute_cdp_cmd('Emulation.setDeviceMetricsOverride', emulation)
            return self.driver.execute_cdp_cmd('Page.setDeviceMetricsOverride', emulation), x

    def set_useragent(self, useragent) -> Dict[str, str or Dict[str, str or bool or List[Dict[str, str]]]]:
        x = self.driver.execute_cdp_cmd('Emulation.setUserAgentOverride', useragent)
        # noinspection PyTypeChecker
        return self.driver.execute_cdp_cmd("Network.setUserAgentOverride", useragent), x

    # BROWSER

    def set_darkmode(self, enabled=True, mobile=True) -> (bool, bool):
        if not mobile and enabled:
            warnings.warn('darkmode might look weird without mobile_view!')
        else:
            return self.driver.execute_cdp_cmd('Emulation.setAutoDarkModeOverride',
                                               {'enabled': enabled})

    # TOUCH

    def set_touchpoints(self, enabled=True, maxpoints=5) -> (bool, int):  # set touch options
        return self.driver.execute_cdp_cmd('Emulation.setTouchEmulationEnabled',
                                           {'enabled': enabled, 'maxTouchPoints': maxpoints})  # already set in options

    def pointer_as_touch(self, mobile, enabled=True) -> (bool, bool):  # (makes code hung)
        if mobile:
            config = 'mobile'
        else:
            config = 'desktop'
        if enabled:
            warnings.warn('Actions execute, but then take long when "EmitTouchEventsForMouse"!')
        return self.driver.execute_cdp_cmd('Emulation.setEmitTouchEventsForMouse', {'enabled': enabled,
                                                                                    'configuration': config})  # executes, but then takes long [maybe check if success?]

    # OVERRIDE JAVASCRIPT

    def evaluate_on_new_document(self, js: str):  # evaluate js on every new page
        identifier = int(
            self.driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {"source": js})["identifier"])
        self.evaluate_on_document_identifiers.update({identifier: js})
        return identifier

    # remove js evaluation  on every new page for specific script
    def remove_evaluate_on_document(self, identifier: int):
        del self.evaluate_on_document_identifiers[identifier]
        return self.driver.execute_cdp_cmd("Page.removeScriptToEvaluateOnNewDocument", {"identifier": str(identifier)})

    # define var.property for javascript
    def define_prop_on_new_document(self, var, prop, val, func='get: ()') -> (str, str, any, str):
        self.evaluate_on_new_document(
            "Object.defineProperty(" + var + ", " + json.dumps(prop) + ", {" + func + " => " + json.dumps(val) + "})")

    # COOKIES

    def get_cookies(self):
        return self.driver.execute_cdp_cmd("Network.getAllCookies", {})

    def get_cookie(self, urls: list = None):
        if urls:
            arg = {"urls": urls}
        else:
            arg = {}
        return self.driver.execute_cdp_cmd("Network.getCookies", arg)

    def add_cookie(self, cookie_dict):
        return self.driver.execute_cdp_cmd("Network.setCookie", cookie_dict)

    def delete_cookie(self, name: str):
        return self.driver.execute_cdp_cmd("Network.deleteCookies", {"name": name})

    def delete_all_cookies(self):
        return self.driver.execute_cdp_cmd("Network.clearBrowserCookies", {})



import setuptools


# noinspection SpellCheckingInspection
def is_collab():
    import sys
    # noinspection PyBroadException
    try:
        # noinspection PyPackageRequirements
        import google.colab
        cimport = True
    except:
        cimport = False
    if ('google.colab' in sys.modules) and cimport:
        return True
    else:
        return False


requirements = ['selenium~=4.6', 'requests~=2.28']

if is_collab():
    requirements.append('PyVirtualDisplay')

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name='selenium_profiles',
    author='Aurin Aegerter',
    author_email='aurinliun@gmx.ch',
    description='Emulate and Automate Chrome using Profiles and Selenium',
    keywords='Selenium,emulation, automation, undetected-chromedriver, webautomation',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/kaliiiiiiiiii/Selenium_Profiles',
    project_urls={
        'Documentation': 'https://github.com/kaliiiiiiiiii/Selenium_Profiles',
        'Bug Reports':
            'https://github.com/kaliiiiiiiiii/Selenium_Profiles/issues',
        'Source Code': 'https://github.com/kaliiiiiiiiii/Selenium_Profiles',
    },
    package_dir={'': 'src'},
    packages=setuptools.find_packages(where='src'),
    classifiers=[
        # see https://pypi.org/classifiers/
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'License :: Free for non-commercial use',
        'Natural Language :: English',
        'Operating System :: Microsoft :: Windows',
        'Framework :: Jupyter',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP :: Browsers',

    ],
    python_requires='>=3.7',
    install_requires=requirements,
    include_package_data=True,
    extras_require={
        'dev': ['check-manifest'],
        # 'test': ['coverage'],
    },
)
